@@ -22,11 +22,12 @@
 #include <sys/types.h>
 #include <unistd.h>
 #include <netdb.h>
+#include <arpa/inet.h>
+
+#include <gcrypt.h>
 
 #include "backend_pgsql.h"
 #include "pam_pgsql.h"
-#include "md5.h"
-#include "sha1.h"
 
 static char *
 crypt_makesalt(pw_scheme scheme);
@@ -204,13 +205,8 @@ pg_execParam(PGconn *conn, PGresult **res,
 	
 	if(rhost != NULL && (hentry = gethostbyname(rhost)) != NULL) {
 		/* Make IP string */
-		raddr = malloc(16);
-		sprintf(raddr, "%d.%d.%d.%d",
-			hentry->h_addr_list[0][0],
-			hentry->h_addr_list[0][1],
-			hentry->h_addr_list[0][2],
-			hentry->h_addr_list[0][3]);
-		raddr[15] = 0;
+		raddr = malloc(INET_ADDRSTRLEN);
+		inet_ntop(AF_INET, hentry->h_addr_list[0], raddr, INET_ADDRSTRLEN);
 	}
 	
 	nparm = expand_query(&command, values, query, service, user, passwd, rhost, raddr);
@@ -264,7 +260,7 @@ backend_authenticate(const char *service, const char *user, const char *passwd,
 			rc = PAM_USER_UNKNOWN;
 		} else {
 			char *stored_pw = PQgetvalue(res, 0, 0);
-			if (!strcmp(stored_pw, (tmp = password_encrypt(options, passwd, stored_pw)))) rc = PAM_SUCCESS; 
+			if (!strcmp(stored_pw, (tmp = password_encrypt(options, user, passwd, stored_pw)))) rc = PAM_SUCCESS;
 			free (tmp);
 		}
 		PQclear(res);
@@ -275,7 +271,7 @@ backend_authenticate(const char *service, const char *user, const char *passwd,
 
 /* private: encrypt password using the preferred encryption scheme */
 char *
-password_encrypt(modopt_t *options, const char *pass, const char *salt)
+password_encrypt(modopt_t *options, const char *user, const char *pass, const char *salt)
 {
 	char *s = NULL;
 
@@ -289,47 +285,49 @@ password_encrypt(modopt_t *options, const char *pass, const char *salt)
 			}
 		break;
 		case PW_MD5: {
-
-			MD5_CTX ctx;
-			char *buf;
-			unsigned char hash[16]; /* 16 is the md5 block size */
+			unsigned char hash[16] = { 0, }; /* 16 is the md5 block size */
 			int i;
+			s = (char *) malloc(33); /* 32 bytes + 1 byte for \0 */
 
-			bzero(&hash, 16);
+			gcry_md_hash_buffer(GCRY_MD_MD5, hash, pass, strlen(pass));
 
-			MD5Init(&ctx);
-			MD5Update(&ctx, (unsigned char const *) pass, strlen(pass));
-			MD5Final(hash, &ctx);
+			for(i = 0; i < sizeof(hash); i++)
+				sprintf(&s[i * 2], "%.2x", hash[i]);
+		}
+		break;
+		case PW_MD5_POSTGRES: {
+			/* This is the md5 variant used by postgres shadow table.
+			cleartext is password||user
+			returned value is md5||md5hash(password||user)
+			*/
+			unsigned char hash[16] = { 0, }; /* 16 is the md5 block size */
+			int i;
+			s = (char *) malloc(33); /* 32 bytes + 1 byte for \0 */
 
-			buf = (char *) malloc(33); /* 32 bytes + 1 byte for \0 */
+			size_t unencoded_length;
+			char *unencoded;
 
-			for(i = 0; i < 16; i++) { 
-				sprintf(&buf[i * 2], "%.2x", hash[i]);
-			}
-			s = buf;
+			unencoded_length = strlen(pass)+strlen(user);
+			unencoded = malloc(unencoded_length+1);
+			sprintf(unencoded, "%s%s", pass, user);
+
+			gcry_md_hash_buffer(GCRY_MD_MD5, hash, unencoded, strlen(unencoded));
+			for(i = 0; i < sizeof(hash); i++)
+				sprintf(&s[i * 2], "%.2x", hash[i]);
+
+			free(unencoded);
 
 		}
 		break;
 		case PW_SHA1: {
-
-			SHA1Context ctx;
-			char *buf;
-			unsigned char hash[20]; /* 20 is the sha1 block size */
+			unsigned char hash[20] = { 0, }; /* 20 is the sha1 block size */
 			int i;
+			s = (char *) malloc(41); /* 40 bytes + 1 byte for \0 */
 
-			bzero(&hash, 20);
-
-			SHA1Reset(&ctx);
-			SHA1Input(&ctx, (unsigned char const *) pass, strlen(pass));
-			SHA1Result(&ctx);
-
-			buf = (char *) malloc(41); /* 40 bytes + 1 byte for \0 */
-
-			for(i = 0; i < 5; i++) { 
-				sprintf(&buf[i * 8], "%.8x", ctx.Message_Digest[i]);
-			}
-			s = buf;
+			gcry_md_hash_buffer(GCRY_MD_SHA1, hash, pass, strlen(pass));
 
+			for(i = 0; i < sizeof(hash); i++)
+				sprintf(&s[i * 2], "%.2x", hash[i]);
 		}
 		break;
 		case PW_CLEAR:
